"""
pages url config
"""

from django.conf.urls import url

from sovtimer import views

app_name: str = "sovtimer"

urlpatterns = [
    url(r"^$", views.dashboard, name="dashboard"),
    # ajax call
    url(
        r"^dashboard_data/$",
        views.dashboard_data,
        name="dashboard_data",
    ),
]



"""
utilities
"""

import logging
import os

from django.conf import settings
from django.utils.functional import lazy
from django.utils.html import format_html


def clean_setting(
    name: str,
    default_value: object,
    min_value: int = None,
    max_value: int = None,
    required_type: type = None,
):
    """cleans the input for a custom setting

    Will use `default_value` if settings does not exit or has the wrong type
    or is outside define boundaries (for int only)

    Need to define `required_type` if `default_value` is `None`

    Will assume `min_value` of 0 for int (can be overriden)

    Returns cleaned value for setting
    """
    if default_value is None and not required_type:
        raise ValueError("You must specify a required_type for None defaults")

    if not required_type:
        required_type = type(default_value)

    if min_value is None and required_type == int:
        min_value = 0

    if not hasattr(settings, name):
        cleaned_value = default_value
    else:
        if (
            isinstance(getattr(settings, name), required_type)
            and (min_value is None or getattr(settings, name) >= min_value)
            and (max_value is None or getattr(settings, name) <= max_value)
        ):
            cleaned_value = getattr(settings, name)
        else:
            logger.warning(
                "You setting for {name} is not valid. Please correct it. "
                "Using default for now: {value}".format(name=name, value=default_value)
            )
            cleaned_value = default_value

    return cleaned_value


# Format for output of datetime for this app
DATETIME_FORMAT = "%Y-%m-%d %H:%M"

format_html_lazy = lazy(format_html, str)


class LoggerAddTag(logging.LoggerAdapter):
    """
    add custom tag to a logger
    """

    def __init__(self, my_logger, prefix):
        super().__init__(my_logger, {})
        self.prefix = prefix

    def process(self, msg, kwargs):
        """
        process log items
        :param msg:
        :param kwargs:
        :return:
        """
        return f"[{self.prefix}] {msg}", kwargs


logger = LoggerAddTag(logging.getLogger(__name__), __package__)


def get_swagger_spec_path() -> str:
    """
    returns the path to the current swagger spec file
    """
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), "swagger.json")


def make_logger_prefix(tag: str):
    """
    creates a function to add logger prefix, which returns tag when used empty
    """
    return lambda text="": "{}{}".format(tag, (": " + text) if text else "")



# Generated by Django 3.1.3 on 2020-11-11 20:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("sovtimer", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="AaSovtimerCampaigns",
            fields=[
                (
                    "campaign_id",
                    models.PositiveBigIntegerField(
                        db_index=True, primary_key=True, serialize=False, unique=True
                    ),
                ),
                ("attackers_score", models.FloatField()),
                ("constellation_id", models.PositiveBigIntegerField()),
                ("defender_id", models.PositiveBigIntegerField()),
                ("defender_score", models.FloatField()),
                ("event_type", models.CharField(max_length=12)),
                ("solar_system_id", models.PositiveBigIntegerField()),
                ("start_time", models.DateTimeField()),
                ("structure_id", models.PositiveBigIntegerField()),
            ],
            options={
                "verbose_name": "Sovereignty Campaign",
                "verbose_name_plural": "Sovereignty Campaigns",
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="AaSovtimerStructures",
            fields=[
                (
                    "structure_id",
                    models.PositiveBigIntegerField(
                        db_index=True, primary_key=True, serialize=False, unique=True
                    ),
                ),
                ("alliance_id", models.PositiveBigIntegerField()),
                ("solar_system_id", models.PositiveBigIntegerField()),
                ("structure_type_id", models.PositiveBigIntegerField()),
                ("vulnerability_occupancy_level", models.FloatField(null=True)),
                ("vulnerable_end_time", models.DateTimeField(null=True)),
                ("vulnerable_start_time", models.DateTimeField(null=True)),
            ],
            options={
                "verbose_name": "Sovereignty Structure",
                "verbose_name_plural": "Sovereignty Structures",
                "default_permissions": (),
            },
        ),
    ]



"""
loading initial data into sovtimer tables
"""

from django.core.cache import cache
from django.core.management.base import BaseCommand
from django.db import transaction

from eveuniverse.core.esitools import is_esi_online
from eveuniverse.models import EveEntity, EveSolarSystem

from sovtimer.models import AaSovtimerCampaigns, AaSovtimerStructures

ESI_SOV_STRUCTURES_CACHE_KEY = "sov_structures_cache"


def get_input(text):
    """
    wrapped input to enable tz import
    """

    return input(text)


class Command(BaseCommand):
    help = "Imports initial data"

    def _import_sov_data(self) -> None:
        """
        import sovereignty data
        :return:
        """

        if not is_esi_online():
            self.stdout.write(
                "ESI is currently offline. Can not start ESI related tasks. Aborting"
            )
            return

        # Import sov structures
        structures_from_esi = AaSovtimerStructures.sov_structures_from_esi()
        if structures_from_esi:
            with transaction.atomic():
                AaSovtimerStructures.objects.all().delete()
                sov_structures = list()

                structure_count = 0

                for structure in structures_from_esi:
                    structure_alliance, _ = EveEntity.objects.get_or_create(
                        id=structure["alliance_id"]
                    )

                    structure_solar_system = EveSolarSystem.objects.get(
                        id=structure["solar_system_id"]
                    )

                    vulnerability_occupancy_level = 1
                    if structure["vulnerability_occupancy_level"]:
                        vulnerability_occupancy_level = structure[
                            "vulnerability_occupancy_level"
                        ]

                    vulnerable_end_time = None
                    if structure["vulnerable_end_time"]:
                        vulnerable_end_time = structure["vulnerable_end_time"]

                    vulnerable_start_time = None
                    if structure["vulnerable_start_time"]:
                        vulnerable_start_time = structure["vulnerable_start_time"]

                    sov_structures.append(
                        AaSovtimerStructures(
                            alliance=structure_alliance,
                            solar_system=structure_solar_system,
                            structure_id=structure["structure_id"],
                            structure_type_id=structure["structure_type_id"],
                            vulnerability_occupancy_level=vulnerability_occupancy_level,
                            vulnerable_end_time=vulnerable_end_time,
                            vulnerable_start_time=vulnerable_start_time,
                        )
                    )

                    structure_count += 1

                AaSovtimerStructures.objects.bulk_create(
                    sov_structures,
                    batch_size=500,
                    ignore_conflicts=True,
                )

                EveEntity.objects.bulk_update_new_esi()

                cache.set(ESI_SOV_STRUCTURES_CACHE_KEY, True, 120)

                self.stdout.write(
                    (
                        "{structure_count} sovereignty structures imported from ESI."
                    ).format(structure_count=structure_count)
                )

        # import sov campaigns
        campaigns_from_esi = AaSovtimerCampaigns.sov_campaigns_from_esi()
        if campaigns_from_esi:
            with transaction.atomic():
                campaigns = list()

                campaign_count = 0

                for campaign in campaigns_from_esi:
                    campaign_current__defender, _ = EveEntity.objects.get_or_create(
                        id=campaign["defender_id"]
                    )

                    campaign_current__solar_system = EveSolarSystem.objects.get(
                        id=campaign["solar_system_id"]
                    )

                    campaign_current__defender_score = campaign["defender_score"]

                    try:
                        campaign_previous = AaSovtimerCampaigns.objects.get(
                            campaign_id=campaign["campaign_id"]
                        )

                        campaign_previous__progress_previous = (
                            campaign_previous.progress_previous
                        )

                        campaign_previous__progress = campaign_previous.defender_score
                        campaign_current__progress_previous = (
                            campaign_previous__progress
                        )

                        if (
                            campaign_previous__progress
                            == campaign_current__defender_score
                        ):
                            campaign_current__progress_previous = (
                                campaign_previous__progress_previous
                            )
                    except AaSovtimerCampaigns.DoesNotExist:
                        campaign_current__progress_previous = (
                            campaign_current__defender_score
                        )

                    campaigns.append(
                        AaSovtimerCampaigns(
                            attackers_score=campaign["attackers_score"],
                            campaign_id=campaign["campaign_id"],
                            defender=campaign_current__defender,
                            defender_score=campaign["defender_score"],
                            event_type=campaign["event_type"],
                            solar_system=campaign_current__solar_system,
                            start_time=campaign["start_time"],
                            structure_id=campaign["structure_id"],
                            progress_current=campaign_current__defender_score,
                            progress_previous=campaign_current__progress_previous,
                        )
                    )

                    campaign_count += 1

                AaSovtimerCampaigns.objects.all().delete()
                AaSovtimerCampaigns.objects.bulk_create(
                    campaigns,
                    batch_size=500,
                    ignore_conflicts=True,
                )

                EveEntity.objects.bulk_update_new_esi()

                self.stdout.write(
                    "{campaign_count} sovereignty campaigns imported from ESI.".format(
                        campaign_count=campaign_count
                    )
                )

    def handle(self, *args, **options):
        """
        ask before running ...
        :param args:
        :param options:
        """

        self.stdout.write(
            "This will start the initial import for SOV campaigns and structures. "
            "It's quite a bit to import, so this might take a moment or two. "
            "Please be patient ..."
        )

        user_input = get_input("Are you sure you want to proceed? (yes/no)?")

        if user_input == "yes":
            self.stdout.write("Starting import. Please stand by.")
            self._import_sov_data()
            self.stdout.write(self.style.SUCCESS("Import complete!"))
        else:
            self.stdout.write(self.style.WARNING("Aborted."))
